package cn.ibizlab.core.extensions.service;

import cn.ibizlab.core.ou.domain.IBZDepartment;
import cn.ibizlab.core.ou.domain.IBZOrganization;
import cn.ibizlab.core.ou.service.IIBZDepartmentService;
import cn.ibizlab.core.ou.service.IIBZOrganizationService;
import cn.ibizlab.util.security.AuthenticationUser;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;

@Service
public class OUModelService
{

	@Autowired
	private IIBZOrganizationService iibzOrganizationService;

	@Autowired
	private IIBZDepartmentService iibzDepartmentService;

	private Map<String, Map<String, Set<String>>> orgmap=null;
	private Object lockOrg=new Object();

	//@Cacheable( value="ibzou-model",key = "'orgmap'")
	public  Map<String, Map<String, Set<String>>> getOrgModel()
	{
		if(orgmap!=null)
			return orgmap;
		synchronized(lockOrg)
		{
			if(orgmap!=null)
				return orgmap;
			Map<String, Map<String, Set<String>>> store=new LinkedHashMap<>();
			Map<String, IBZOrganization> orgset=new LinkedHashMap<>();
			List<IBZOrganization> listOrg=iibzOrganizationService.list();
			for(IBZOrganization org:listOrg)
			{
				Map<String, Set<String>> map=new HashMap<>();
				Set<String> parent=new LinkedHashSet<String>();
				if(!StringUtils.isEmpty(org.getParentorgid()))
					parent.add(org.getParentorgid());
				map.put("parent",parent);
				Set<String> sub=new LinkedHashSet<String>();
				sub.add(org.getOrgid());
				map.put("sub",sub);
				store.put(org.getOrgid(),map);
				orgset.put(org.getOrgid(),org);
			}

			for(IBZOrganization org:listOrg)
			{
				loopOrg(org,orgset,store);
			}

			for(IBZOrganization org:listOrg)
			{
				for(String sub:store.get(org.getOrgid()).get("sub"))
				{
					if(!org.getOrgid().equals(sub))
						store.get(sub).get("parent").add(org.getOrgid());
				}
			}

			orgmap=store;
		}

		return orgmap;
	}

	public void loopOrg(IBZOrganization org,Map<String, IBZOrganization> orgset,Map<String, Map<String, Set<String>>> store)
	{
		if(!StringUtils.isEmpty(org.getParentorgid()))
		{
			if(store.containsKey(org.getParentorgid()))
			{
				store.get(org.getParentorgid()).get("sub").add(org.getOrgid());
				loopOrg(orgset.get(org.getParentorgid()),orgset,store);
			}
		}

	}



	private Map<String, Map<String, Set<String>>> deptmap=null;

	private Object lockDept=new Object();

	//@Cacheable( value="ibzou-model",key = "'deptmap'")
	public  Map<String, Map<String, Set<String>>> getDeptModel(Map<String, Map<String, Set<String>>> orgstore)
	{
		if(deptmap!=null)
			return deptmap;
		synchronized(lockDept)
		{
			if(deptmap!=null)
				return deptmap;
			if(orgstore==null)
				orgstore=this.getOrgModel();
			Map<String, Map<String, Set<String>>> store=new LinkedHashMap<>();
			Map<String, IBZDepartment> deptset=new LinkedHashMap<>();

			Map<String, Set<String>> bcmap=new HashMap<>();

			List<IBZDepartment> listDept=iibzDepartmentService.list();
			for(IBZDepartment dept:listDept)
			{
				Map<String, Set<String>> map=new HashMap<>();
				Set<String> parent=new LinkedHashSet<String>();
				if(!StringUtils.isEmpty(dept.getParentdeptid()))
					parent.add(dept.getParentdeptid());
				map.put("parent",parent);
				Set<String> sub=new LinkedHashSet<String>();
				sub.add(dept.getDeptid());
				map.put("sub",sub);
				store.put(dept.getDeptid(),map);
				deptset.put(dept.getDeptid(),dept);

				String bc=dept.getBcode();
				if(!StringUtils.isEmpty(bc))
				{
					bc=bc.replace(";",",").replace("；",",").replace("，",",");
					String[] bcs=bc.split(",");
					for(String strbc:bcs)
					{
						strbc=strbc.trim();
						if(!StringUtils.isEmpty(strbc))
						{
							if(!bcmap.containsKey(strbc))
							{
								Set<String> bcset=new HashSet<>();
								bcmap.put(strbc,bcset);
							}
							bcmap.get(strbc).add(dept.getDeptid()+":"+dept.getOrgid());
						}
					}

				}

			}

			for(IBZDepartment dept:listDept)
			{
				loopDept(dept,deptset,store);
			}

			for(IBZDepartment dept:listDept)
			{
				for(String sub:store.get(dept.getDeptid()).get("sub"))
				{
					if(!dept.getDeptid().equals(sub))
						store.get(sub).get("parent").add(dept.getDeptid());
				}
			}

			for(IBZDepartment dept:listDept)
			{
				String orgid=dept.getOrgid();
				if(StringUtils.isEmpty(orgid)||(!orgstore.containsKey(orgid)))
					continue;

				String bc=dept.getBcode();
				if(!StringUtils.isEmpty(bc))
				{
					bc = bc.replace(";", ",").replace("；", ",").replace("，", ",");
					String[] bcs = bc.split(",");
					for (String strbc : bcs)
					{
						strbc = strbc.trim();
						if (!StringUtils.isEmpty(strbc))
						{
							if(bcmap.containsKey(strbc))
							{
								Set<String> bcset=bcmap.get(strbc);
								for(String bcstring:bcset)
								{
									String[] bcstringarr=bcstring.split(":");
									if(bcstringarr.length==2)
									{
										String bcdept=bcstringarr[0];
										String bcorg=bcstringarr[1];
										for(String porg:orgstore.get(orgid).get("parent"))
										{
											if(bcorg.equals(porg))
												store.get(dept.getDeptid()).get("parent").add(bcdept);
										}
										for(String sorg:orgstore.get(orgid).get("sub"))
										{
											if(bcorg.equals(sorg))
												store.get(dept.getDeptid()).get("sub").add(bcdept);
										}
									}
								}
							}
						}
					}
				}
			}

			deptmap=store;
		}

		return deptmap;
	}

	public void loopDept(IBZDepartment dept,Map<String, IBZDepartment> deptset,Map<String, Map<String, Set<String>>> store)
	{
		if(!StringUtils.isEmpty(dept.getParentdeptid()))
		{
			if(store.containsKey(dept.getParentdeptid()))
			{
				store.get(dept.getParentdeptid()).get("sub").add(dept.getDeptid());
				loopDept(deptset.get(dept.getParentdeptid()),deptset,store);
			}
		}

	}


	//@CacheEvict( value="ibzou-model",allEntries = true)
	public synchronized void refreshModel()
	{

		synchronized (lockDept)
		{
			this.deptmap = null;
		}
		synchronized (lockOrg)
		{
			this.orgmap = null;
		}
	}

}
